Package: bbmle
Title: Tools for General Maximum Likelihood Estimation
Description: Methods and functions for fitting maximum likelihood models in R. This package modifies and extends the 'mle' classes in the 'stats4' package.
Version: 1.0.25.1
Authors@R: c(person("Ben","Bolker",email="bolker@mcmaster.ca",role=c("aut","cre"),
	   comment=c(ORCID="0000-0002-2127-0443")),
	   person("R Development Core Team",role=c("aut")),
	   person("Iago Giné-Vázquez", role=c("ctb"))
	  )
Depends: R (>= 3.0.0), stats4
Imports: stats, numDeriv, lattice, MASS, methods, bdsmatrix, Matrix,
        mvtnorm
Suggests: emdbook, rms, ggplot2, RUnit, MuMIn, AICcmodavg, Hmisc,
        optimx (>= 2013.8.6), knitr, testthat
VignetteBuilder: knitr
BuildVignettes: yes
License: GPL
URL: https://github.com/bbolker/bbmle
Collate: 'mle2-class.R' 'mle2-methods.R' 'mle.R' 'confint.R'
        'predict.R' 'profile.R' 'update.R' 'dists.R' 'IC.R' 'slice.R'
        'impsamp.R' 'TMB.R'
RoxygenNote: 7.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-12-08 23:45:52 UTC; bolker
Author: Ben Bolker [aut, cre] (<https://orcid.org/0000-0002-2127-0443>),
  R Development Core Team [aut],
  Iago Giné-Vázquez [ctb]
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Repository: CRAN
Date/Publication: 2023-12-09 01:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 06:18:41 UTC; unix
